@@ -10,6 +10,8 @@ class Widget(Component, _Widget):
     """
     Base class for widget components.
     Widget components are put directly on the main window, hidden.
+    
+    @since: 0.96.4
     """
 
 
@@ -18,3 +20,13 @@ class Widget(Component, _Widget):
         Component.__init__(self)
         _Widget.__init__(self)
         self.set_visible(False)
+        
+        
+    def set_flag(*args):
+    
+        pass
+        
+        
+    def set_title(*args):
+    
+        pass
