@@ -61,6 +61,14 @@ _MODE_NORMAL = 0
 _MODE_SELECT = 1
 
 
+# it's useful to swap the mapping of the increment and decrement keys in
+# portrait mode
+_PORTRAIT_MODE_KEYSWAP = {
+    "F7": "F8",
+    "F8": "F7"
+}
+
+
 class Navigator(Component, Window):
     """
     Navigator dialog for browsing media.
@@ -90,7 +98,7 @@ class Navigator(Component, Window):
         self.__window_size = (0, 0)
         
         self.__is_searching = False
-        self.__search_term = ""
+        self.__filter_term = ""
         
         self.__key_hold_down_timestamp = 0
         self.__skip_letter = False
@@ -98,9 +106,13 @@ class Navigator(Component, Window):
         # scheduler for creating thumbnails one by one
         self.__tn_scheduler = ItemScheduler()
         
+        # whether we are shutting down
+        self.__is_shutdown = False
+        
     
         Component.__init__(self)
         Window.__init__(self, Window.TYPE_TOPLEVEL)
+        self.set_flag(windowflags.CATCH_VOLUME_KEYS, True)
         self.connect_key_pressed(self.__on_key_press)
         self.connect_closed(self.__on_close_window)
 
@@ -254,6 +266,7 @@ class Navigator(Component, Window):
         #self.__browser.render()
 
 
+    """
     def _visibility_changed(self):
         
         Window._visibility_changed(self)
@@ -262,11 +275,56 @@ class Navigator(Component, Window):
             
         else:
             self.__tn_scheduler.halt()
+    """
 
 
     def __on_key_press(self, keycode):
 
-        self.call_service(msgs.INPUT_SVC_SEND_KEY, keycode, True)
+        w, h = self.get_size()
+        if (w < h and
+              mb_config.portrait_swap_volume() and
+              keycode in _PORTRAIT_MODE_KEYSWAP):
+            keycode = _PORTRAIT_MODE_KEYSWAP[keycode]
+
+        handled = self.call_service(msgs.INPUT_SVC_SEND_KEY, keycode, True)
+        if (not handled):
+            """
+            if (len(keycode) == 1 and ord(keycode) > 31):
+                cnt = 0
+                for item in self.__browser.get_items():
+                    if (item.get_name().upper().startswith(keycode.upper())):
+                        self.__browser.scroll_to_item(cnt)
+                        break
+                    cnt += 1
+                #end for
+            """    
+            if (len(keycode) == 1 and ord(keycode) > 31):
+                self.__filter_term += keycode
+                self.__update_filter()
+            elif (keycode == "BackSpace" and self.__filter_term):
+                self.__filter_term = self.__filter_term[:len(self.__filter_term) - 1]
+                self.__update_filter()
+
+
+    def __update_filter(self):
+
+        def filter_func(item):
+            return self.__filter_term.upper() in item.get_name().upper() + "#" + \
+                                                 item.get_file().info.upper()
+
+        if (self.__filter_term):
+            self.__browser.set_filter(filter_func)
+            self.__browser.set_message("Filter: %s (%d matches)" \
+                                       % (self.__filter_term,
+                                          self.__browser.count_items()))
+        else:
+            self.__browser.set_filter()
+            self.__browser.set_message("")
+                
+        self.__browser.invalidate()
+        self.__browser.render()
+
+
 
 
     def __on_close_window(self):
@@ -368,6 +426,9 @@ class Navigator(Component, Window):
         #self.set_title(f.name)
         self.set_flag(windowflags.BUSY, True)
         
+        self.__browser.set_filter()
+        self.__filter_term = ""
+        
         self.__update_layout()
         self.__update_menu()
         self.render()
@@ -381,21 +442,27 @@ class Navigator(Component, Window):
         else:
             self.__browser.set_click_behavior(
               self.__browser.CLICK_BEHAVIOR_SINGLE)
-        
+                
 
     def __on_progress_folder(self, f, c):
 
-        if (c.icon): return
+        if (f == self.__play_folder and not c.mimetype.endswith("-folder")):
+            if (not c in self.__play_files):
+                self.__play_files.append(c)
+            if (not c in self.__random_files):
+                self.__random_files.append(c)
 
-        item = self.__browser.get_items()[-1]
-        thumbpath, is_final = \
-          self.call_service(msgs.THUMBNAIL_SVC_LOOKUP_THUMBNAIL, c)
+        if (not c.icon):
+            item = self.__browser.get_items()[-1]
+            thumbpath, is_final = \
+              self.call_service(msgs.THUMBNAIL_SVC_LOOKUP_THUMBNAIL, c)
 
-        item.set_icon(thumbpath)
-        
-        if (not is_final):
-            #print "SCHEDULING THUMBNAIL", c
-            self.__tn_scheduler.add(item, c)
+            item.set_icon(thumbpath)
+            
+            if (not is_final):
+                #print "SCHEDULING THUMBNAIL", c
+                self.__tn_scheduler.add(item, c)
+        #end if
 
 
     def __on_complete_folder(self, f):
@@ -414,14 +481,17 @@ class Navigator(Component, Window):
 
         if (self.is_visible()):
             self.__tn_scheduler.resume()
-
+            
 
     def __update_thumbnail(self, item, thumbpath):
     
         #pass
         item.set_icon(thumbpath)
-        idx = self.__browser.get_items().index(item)
-        self.__browser.invalidate_item(idx)
+        try:
+            idx = self.__browser.get_items().index(item)
+            self.__browser.invalidate_item(idx)
+        except:
+            pass
 
 
     def __on_load_thumbnail(self, item, f):
@@ -431,7 +501,14 @@ class Navigator(Component, Window):
             if (thumbpath):
                 self.__update_thumbnail(item, thumbpath)
             
-            #if (self.is_visible()):
+            # priorize visible items
+            top_idx = self.__browser.get_item_at(0, 0)
+            if (top_idx != -1):
+                items = self.__browser.get_items()
+                if (top_idx < len(items)):
+                    self.__tn_scheduler.priorize(items[top_idx:top_idx + 12])
+            #end if
+            
             self.__tn_scheduler.resume()
             #print "RESUMING SCHEDULER"
     
@@ -542,21 +619,22 @@ class Navigator(Component, Window):
 
             # update set of play files
             self.__current_file = f
-            self.__browser.hilight_file(f)
-            self.__browser.render()
 
             folder = self.__browser.get_current_folder()
             if (is_manual and folder != self.__play_folder):
                 self.__play_folder = folder
                 self.__play_files = [ fl for fl in self.__browser.get_files()
                                       if not fl.mimetype.endswith("-folder") ]
-                self.__filter_play_files()
                 self.__random_files = []
+                logging.debug("[navigator] clearing random items")
 
 
 
     def __go_previous(self):
 
+        now = time.time()
+        self.__filter_play_files()
+        
         try:
             idx = self.__play_files.index(self.__current_file)
         except ValueError:
@@ -566,9 +644,14 @@ class Navigator(Component, Window):
             next_item = self.__play_files[idx - 1]
             self.__load_file(next_item, False)
             
+        logging.profile(now, "[navigator] loaded previous item")
+            
             
     def __go_next(self):
 
+        now = time.time()
+        self.__filter_play_files()
+        
         repeat_mode = mb_config.repeat_mode()
         shuffle_mode = mb_config.shuffle_mode()
         
@@ -598,6 +681,8 @@ class Navigator(Component, Window):
             elif (shuffle_mode == mb_config.SHUFFLE_MODE_ALL):
                 self.__play_shuffled(True)
             
+        logging.profile(now, "[navigator] loaded next item")
+        
 
     def __play_same(self):
     
@@ -637,11 +722,14 @@ class Navigator(Component, Window):
 
         if (not self.__random_files):
             self.__random_files = self.__play_files[:]
+            logging.debug("[navigator] initialising random items (%d items)",
+                          len(self.__random_files))
 
         idx = random.randint(0, len(self.__random_files) - 1)
         next_item = self.__random_files.pop(idx)
-        print self.__random_files
-        print idx, "->", next_item
+        logging.debug("[navigator] picking random item %d of %d: %s",
+                      idx, len(self.__random_files), str(next_item))
+
         self.__load_file(next_item, False)
         
         return True
@@ -653,6 +741,10 @@ class Navigator(Component, Window):
         from the list.
         """
         
+        if (not self.__play_files): return
+        
+        now = time.time()
+        
         size = len(self.__play_files)
         img_size = len([f for f in self.__play_files
                         if f.mimetype in mimetypes.get_image_types() ])
@@ -662,6 +754,12 @@ class Navigator(Component, Window):
             self.__play_files = [ f for f in self.__play_files
                              if not f.mimetype in mimetypes.get_image_types() ]
 
+            self.__random_files = [ f for f in self.__random_files
+                                    if f in self.__play_files ]
+
+        logging.profile(now, "[navigator] filtered items to play " \
+                             "(removed cover art)")
+
 
     def __show_dialog(self, name):
         """
@@ -676,6 +774,18 @@ class Navigator(Component, Window):
         #end if
 
 
+    def __save_state(self):
+
+        # save state for next start
+        path = [ f.full_path for f in self.__browser.get_path_stack() ]
+        play_files = [ f.full_path for f in self.__play_files ]
+        
+        state.save(_STATEFILE, path,
+                               play_files,
+                               self.__play_folder and self.__play_folder.full_path or "",
+                               self.__current_file and self.__current_file.full_path or "")
+
+
     def render_this(self):
     
         w, h = self.get_size()
@@ -687,6 +797,23 @@ class Navigator(Component, Window):
         if (self.__arr.is_visible()):
             Window.render_this(self)
             
+        elif (self.__is_shutdown):
+            x, y = self.get_screen_pos()
+            screen = self.get_screen()
+        
+            screen.fill_area(x, y, w, h, theme.color_mb_background)
+            screen.draw_centered_text(values.NAME + " " + values.VERSION,
+                                      theme.font_mb_headline,
+                                      x, h / 2 - 30, w, 30, theme.color_mb_text)
+            screen.draw_centered_text(values.COPYRIGHT,
+                                      theme.font_mb_plain,
+                                      x, h / 2, w, 30, theme.color_mb_text)
+            screen.draw_centered_text("Exiting...",
+                                      theme.font_mb_plain,
+                                      x, h - 80, w, 20, theme.color_mb_text)
+            screen.fit_pixbuf(theme.mb_logo,
+                              w - 120, h - 120, 120, 120)
+
         else:
             x, y = self.get_screen_pos()
             screen = self.get_screen()
@@ -724,6 +851,8 @@ class Navigator(Component, Window):
 
     def handle_COM_EV_APP_STARTED(self):
     
+        logging.profile(values.START_TIME, "[app] startup complete")
+    
         # load state
         try:
             path, play_files, play_folder, current_file = state.load(_STATEFILE)
@@ -747,7 +876,8 @@ class Navigator(Component, Window):
                                                     current_file)
             
         except:
-            pass
+            logging.warning("could not restore navigator state:\n%s",
+                            logging.stacktrace())
         
     
         self.__arr.set_visible(True)
@@ -760,21 +890,18 @@ class Navigator(Component, Window):
             f = self.call_service(msgs.CORE_SVC_GET_FILE,
                                   "adhoc://" + File.pack_path("/", values.uri,
                                                               mimetype))
-            print f
             self.__load_file(f, True)
         #end if
 
     
     def handle_COM_EV_APP_SHUTDOWN(self):
 
-        # save state for next start
-        path = [ f.full_path for f in self.__browser.get_path_stack() ]
-        play_files = [ f.full_path for f in self.__play_files ]
+        self.__is_shutdown = True
+        self.render()
         
-        state.save(_STATEFILE, path,
-                               play_files,
-                               self.__play_folder and self.__play_folder.full_path or "",
-                               self.__current_file and self.__current_file.full_path or "")
+        while (gtk.events_pending()): gtk.main_iteration(True)
+        self.__save_state()
+
 
 
     def handle_CORE_EV_DEVICE_ADDED(self, ident, device):
@@ -801,18 +928,26 @@ class Navigator(Component, Window):
 
     def handle_CORE_EV_FOLDER_INVALIDATED(self, folder):
 
+        logging.debug("[navigator] invalidating folder %s", str(folder))
+        
         if (self.is_visible()):
             self.__browser.invalidate_folder(folder)
+            
         if (folder and folder == self.__play_folder):
             prev_play_files = self.__play_files[:]
             self.__play_files = [ fl for fl in folder.get_children()
                                   if not fl.mimetype.endswith("-folder") ]
-            self.__filter_play_files()
             # remove from random files what's no longer there and add to random
             # files what's new in play files
+            logging.debug("[navigator] updating random items after folder " \
+                          "invalidation")
+            l = len(self.__random_files)
             self.__random_files = [ f for f in self.__play_files
                                     if f in self.__random_files or
                                        f not in prev_play_files ]
+            logging.debug("[navigator] previously %d random items, now %d",
+                          l, len(self.__random_files))
+           
 
 
     def handle_CORE_EV_DEVICE_REMOVED(self, dev_id):
@@ -820,31 +955,6 @@ class Navigator(Component, Window):
         folder = self.__browser.get_current_folder()
         if (folder.device_id == dev_id):
             self.__browser.go_root()
-        
-
-
-    def handle_CORE_ACT_SEARCH_ITEM(self, key):
-    
-        if (self.is_active()):
-            #self.__browser.set_message("Search: " + key)
-            if (key != self.__search_term):
-                self.__browser.set_cursor(1)
-            idx = self.__browser.search(key, 1)
-            if (idx != -1):
-                self.__browser.set_cursor(idx)
-                self.__browser.scroll_to_item(idx)
-            self.__is_searching = True
-            self.__search_term = key
-        #end if
-
-
-    def handle_CORE_EV_SEARCH_CLOSED(self):
-    
-        #self.__browser.set_message("")
-        self.__browser.search("", 1)
-        self.__browser.render()
-        self.__is_searching = False
-        self.__search_term = ""
 
 
     def handle_ASR_ACT_ENABLE(self, value):
@@ -865,16 +975,28 @@ class Navigator(Component, Window):
 
 
     def handle_MEDIA_EV_LOADED(self, player, f):
-    
+        
         if (not self.__now_playing.is_visible()):
             self.__update_layout()
-            self.render()
+    
+        self.__browser.hilight_file(f)
+        self.__browser.render()
+
                     
 
     def handle_MEDIA_ACT_LOAD_URI(self, uri, mimetype):
     
         self.__load_uri(uri, mimetype)
-                    
+
+
+    def handle_MEDIA_ACT_CHANGE_PLAY_FOLDER(self, folder):
+    
+        self.__play_folder = folder
+        self.__play_files = [ fl for fl in self.__browser.get_files()
+                              if not fl.mimetype.endswith("-folder") ]
+        logging.debug("[navigator] clearing random items")
+        self.__random_files = []
+   
     
     def handle_MEDIA_ACT_PREVIOUS(self):
     
@@ -897,3 +1019,9 @@ class Navigator(Component, Window):
         if (self.is_visible()):
             self.__browser.move(0, 80)
 
+
+    def handle_INPUT_EV_KEY(self, key, pressed):
+    
+        if (self.is_visible()):
+            print "GOT KEY", key
+
