@@ -6,6 +6,7 @@ from ui.ImageButton import ImageButton
 from ui.ProgressBar import ProgressBar
 from ui.Slider import Slider
 from ui.layout import Arrangement
+from ui import windowflags
 from mediabox import media_bookmarks
 import mediaplayer
 import platforms
@@ -136,7 +137,8 @@ class VideoPlayer(Player):
                 self.__btn_star.set_visible(False)
             self.__arr.set_xml(_PORTRAIT_ARRANGEMENT)           
         else:
-            self.__btn_star.set_visible(True)
+            if (not self.__is_fullscreen):
+                self.__btn_star.set_visible(True)
             self.__arr.set_xml(_LANDSCAPE_ARRANGEMENT)
 
 
@@ -252,6 +254,12 @@ class VideoPlayer(Player):
             self.__volume = v
 
 
+    def __on_change_player_aspect_ratio(self, ctx_id, r):
+    
+        if (ctx_id == self.__context_id and self.__player):
+            self.__screen.set_aspect_ratio(r)
+        
+
     def __on_change_bookmark(self):
     
         media_bookmarks.set_bookmarks(self.__current_file,
@@ -294,10 +302,25 @@ class VideoPlayer(Player):
             self.__toolbar.set_visible(True)
             self.__btn_star.set_visible(True)
 
-        self.emit_message(msgs.UI_ACT_FULLSCREEN, self.__is_fullscreen)
         self.__update_layout()
-        self.render()
+        self.emit_message(msgs.UI_ACT_FULLSCREEN, self.__is_fullscreen)
+        #self.render()
+
+
+    def _visibility_changed(self):
+    
+        # force video to landscape mode on Maemo
+        win = self.get_window()
+        if (win and platforms.MAEMO5):
+            if (self.is_visible()):
+                logging.debug("[videoplayer] forcing landscape mode")
+                self.emit_message(msgs.ASR_ACT_ENABLE, False)
+                self.emit_message(msgs.ASR_EV_LANDSCAPE)
             
+            else:
+                self.emit_message(msgs.ASR_ACT_RESTORE)
+        #end if
+
 
     def render_this(self):
     
@@ -315,10 +338,10 @@ class VideoPlayer(Player):
         
         if (w < h):
             self.__screen.set_visible(False)
-            screen.draw_centered_text("Video cannot be displayed in\n" \
-                                      "portrait mode.",
-                                      theme.font_mb_plain,
-                                      0, h / 2 - 80, w, 0, theme.color_mb_text)
+            #screen.draw_centered_text("Video cannot be displayed in\n" \
+            #                          "portrait mode.",
+            #                          theme.font_mb_plain,
+            #                          0, h / 2 - 80, w, 0, theme.color_mb_text)
         else:
             self.__screen.set_visible(True)
 
@@ -350,6 +373,7 @@ class VideoPlayer(Player):
         self.__player.connect_status_changed(self.__on_status_changed)
         self.__player.connect_position_changed(self.__on_update_position)
         self.__player.connect_volume_changed(self.__on_change_player_volume)
+        self.__player.connect_aspect_changed(self.__on_change_player_aspect_ratio)
         self.__player.connect_error(self.__on_error)
         
         self.__player.set_window(self.__screen.get_xid())
@@ -382,7 +406,7 @@ class VideoPlayer(Player):
             self.emit_message(msgs.MEDIA_EV_LOADED, self, f)
         except:
             self.__progress.set_message("Error")
-            logging.error("error loading media file: %s\n%s",
+            logging.error("[videoplayer] error loading media file: %s\n%s",
                           f, logging.stacktrace())
 
 
@@ -408,41 +432,32 @@ class VideoPlayer(Player):
             self.__player.stop()
 
 
-    def handle_ASR_EV_LANDSCAPE(self):
+    def handle_MEDIA_ACT_SEEK(self, pos):
+    
+        if (self.__player and self.is_player_active()):
+            self.__player.seek(pos)
 
-        self.__screen.set_visible(True)
-        self.render()
-        
-    """
-    def handle_ASR_EV_PORTRAIT(self):
-
-        self.__screen.set_visible(False)
-        self.render()
-        
         
     def handle_INPUT_EV_FULLSCREEN(self, pressed):
 
         if (self.is_visible()):
             self.__toggle_fullscreen()
-    """
 
 
     def handle_INPUT_EV_VOLUME_UP(self, pressed):
 
-        if (self.is_visible()):
-            self.__volume = min(100, self.__volume + 5)
-            self.__volume_slider.set_value(self.__volume / 100.0)
-            if (self.__player):
-                self.__player.set_volume(self.__volume)
+        self.__volume = min(100, self.__volume + 5)
+        self.__volume_slider.set_value(self.__volume / 100.0)
+        if (self.__player):
+            self.__player.set_volume(self.__volume)
         
         
     def handle_INPUT_EV_VOLUME_DOWN(self, pressed):
 
-        if (self.is_visible()):
-            self.__volume = max(0, self.__volume - 5)
-            self.__volume_slider.set_value(self.__volume / 100.0)
-            if (self.__player):
-                self.__player.set_volume(self.__volume)
+        self.__volume = max(0, self.__volume - 5)
+        self.__volume_slider.set_value(self.__volume / 100.0)
+        if (self.__player):
+            self.__player.set_volume(self.__volume)
 
 
     def handle_COM_EV_APP_SHUTDOWN(self):
