@@ -3,14 +3,18 @@ delayed = True
 def get_classes():
 
     import platforms
+    import os
     from DSPSemaphore import DSPSemaphore
     from VideoPlayer import VideoPlayer
     from VideoInspector import VideoInspector
     classes = [DSPSemaphore, VideoPlayer, VideoInspector]
     
-    if (platforms.PLATFORM == platforms.MAEMO5):
-        from OrgFreeDesktopThumbnailer import OrgFreeDesktopThumbnailer
-        classes.append(OrgFreeDesktopThumbnailer)
+    if (platforms.MAEMO5):
+        from OrgFreeDesktopVideoThumbnailer import OrgFreeDesktopVideoThumbnailer
+        classes.append(OrgFreeDesktopVideoThumbnailer)
+    elif (os.path.exists("/usr/bin/mplayer")):
+        from MPlayerThumbnailer import MPlayerThumbnailer
+        classes.append(MPlayerThumbnailer)
     else:
         from VideoThumbnailer import VideoThumbnailer
         classes.append(VideoThumbnailer)
