@@ -47,6 +47,11 @@ class MediaItem(Item):
         return True
 
 
+    def get_name(self):
+    
+        return self.__file.name
+
+
     def get_file(self):
     
         return self.__file
@@ -99,8 +104,9 @@ class MediaItem(Item):
         else:
             pmap = self.__render_normal()
 
-        # copy to the given canvas
-        cnv.copy_buffer(pmap, 0, 0, x, y, w, h)
+        # copy to the given canvas (may be None for just prerendering to cache)
+        if (cnv):
+            cnv.copy_buffer(pmap, 0, 0, x, y, w, h)
     
             
     def __render_compact(self):
@@ -242,12 +248,14 @@ class MediaItem(Item):
         Loads the icon pixbuf and its frame, if any.
         """
 
+        """
         # try to retrieve from icon cache
         if (path and path.startswith("/") and path in self.__thumbnail_cache):
             mtime, pbuf = self.__thumbnail_cache[path]
             if (os.path.getmtime(path) <= mtime):
                 return pbuf
         #end if
+        """
 
         if (path):
             try:
@@ -285,9 +293,11 @@ class MediaItem(Item):
         if (frm):
             pixbuftools.draw_pbuf(pbuf, frm, 0, 0)
 
+        """
         self.__trim_cache()
         if (path and path.startswith("/")):
             self.__thumbnail_cache[path] = (os.path.getmtime(path), pbuf)
+        """
 
         return pbuf
 
