@@ -1,5 +1,6 @@
 from MediaItem import MediaItem
 from ui.itemview.ThumbableGridView import ThumbableGridView
+from ui.Pixmap import Pixmap
 from ui.dialog import OptionDialog
 from ui.Pixmap import text_extents
 from theme import theme
@@ -8,6 +9,7 @@ from utils import logging
 
 import time
 import gobject
+import gtk
 
 
 _STATUS_OK = 0
@@ -37,6 +39,10 @@ class StorageBrowser(ThumbableGridView):
         # list of tuples: (path, status)
         self.__path_stack = []
 
+        # queue of items for prerendering
+        self.__items_to_prerender = []
+        self.__prerender_handler = None
+
         # the currently hilighted file
         self.__hilighted_file = None
 
@@ -134,6 +140,7 @@ class StorageBrowser(ThumbableGridView):
                          theme.color_list_message)
 
 
+    """
     def __render_search_box(self, screen):
 
         if (not self.__search_term): return
@@ -157,17 +164,10 @@ class StorageBrowser(ThumbableGridView):
         
         screen.draw_pixbuf(theme.mb_list_top, 0, 0, w, 32, True)
         screen.draw_pixbuf(theme.mb_list_bottom, 0, h - 32, w, 32, True)
-        
+    """
+     
 
     def __on_item_clicked(self, item):
-    
-        #idx = self.get_items().index(item)
-    
-        #self.set_cursor(idx)
-        #self.set_hilight(idx)
-        #self.invalidate_item(idx)
-        #self.invalidate()
-        #self.render()
 
         f = item.get_file()
         
@@ -214,6 +214,22 @@ class StorageBrowser(ThumbableGridView):
         folder = self.get_current_folder()
         folder.shift_file(pos, amount)
 
+
+    def __prerender_item(self, t):
+        """
+        Takes an item from the queue and prerenders it.
+        """
+        
+        if (self.__items_to_prerender):
+            item = self.__items_to_prerender.pop(0)
+            #print "prerendering", item.get_name()
+            item.render_at(None, 0, 0)
+            return True
+        else:
+            self.__prerender_handler = None
+            logging.profile(t, "[browser] finished prerendering items")
+            return False
+
         
     def set_root_device(self, device):
         """
@@ -223,7 +239,6 @@ class StorageBrowser(ThumbableGridView):
         """
         
         root = device.get_root()
-        print "SET ROOT DEVICE", device, root
         self.load_folder(root, self.GO_NEW)
 
 
@@ -496,6 +511,9 @@ class StorageBrowser(ThumbableGridView):
                 reload_only = True
         #end if
 
+        if (not reload_only):
+            self.set_filter()
+
         # is a full reload required?
         if (reload_only):
             full_reload = True
@@ -519,6 +537,7 @@ class StorageBrowser(ThumbableGridView):
         self.switch_item_set(folder.full_path)
         self.set_cursor(-1)
         self.emit_event(self.EVENT_FOLDER_BEGIN, folder)
+        self.__items_to_prerender = []
         
         if (folder.folder_flags & folder.ITEMS_UNSORTED):
             self.set_letter_enabled(False)
@@ -528,6 +547,7 @@ class StorageBrowser(ThumbableGridView):
         if (full_reload):
             # reload list
             self.clear_items()
+            self.invalidate()
             
         else:
             # don't reload list
@@ -535,12 +555,15 @@ class StorageBrowser(ThumbableGridView):
 
         # animate
         if (direction == self.GO_CHILD):
-            self.fx_slide_left()
+            #self.fx_slide_left()
+            pass
         elif (direction == self.GO_PARENT):
-            self.fx_slide_right()
+            #self.fx_slide_right()
+            pass
         else:
             #self.render()
             pass
+        self.render()
 
         # load remaining items
         loading_status = self.__path_stack[-1][1]
@@ -550,6 +573,10 @@ class StorageBrowser(ThumbableGridView):
         else:
             self.emit_event(self.EVENT_FOLDER_COMPLETE, folder)
 
+        if (not self.__prerender_handler):
+            self.__prerender_handler = gobject.idle_add(self.__prerender_item,
+                                                        time.time())
+
         # now is a good time to collect garbage
         #import gc; gc.collect()    
       
@@ -564,6 +591,7 @@ class StorageBrowser(ThumbableGridView):
             # abort if the user has changed the directory inbetween
             if (token != self.__token): return False
 
+            profile_now = time.time()
             if (f):
                 #self.set_message("Loading") # (%d items)" % len(self.get_files()))
                 entries.append(f)
@@ -595,24 +623,32 @@ class StorageBrowser(ThumbableGridView):
             #end if
 
             # give visual feedback while loading the visible part of a folder
-            if (not f or len(entries) == 12):
+            if (len(entries) == 12 or not f): #not f or len(entries) == 12):
+                profile_now3 = time.time()
                 self.invalidate()
                 self.render()
+                logging.profile(profile_now3, "[browser] rendered list view")
+                #while (gtk.events_pending()):
+                #    gtk.main_iteration(True)
             #end if
 
-            #"""
             # don't block UI while loading non-local folders
-            if (time.time() > open_time + 2):
-                import gtk
+            #t = int((time.time() - open_time) * 10)
+            if (time.time() > open_time + 3 and len(entries) % 2 == 0):
+                profile_now2 = time.time()
                 while (gtk.events_pending()):
                     gtk.main_iteration(False)
-            #"""
+                logging.profile(profile_now2, "[browser] processed UI events")
 
             if (not f):
                 # last item has been reached
+                logging.profile(open_time, "[browser] loaded %d items",
+                                len(entries))
                 return False
             else:
                 # continue loading next item
+                #logging.profile(profile_now, "[browser] added %dth item",
+                #                len(entries))
                 return True
 
 
@@ -646,10 +682,18 @@ class StorageBrowser(ThumbableGridView):
     
         self.append_item(item)
 
-        idx = len(self.get_items()) - 1
+        l = len(self.get_items())
+        idx = l - 1
 
         # hilight currently selected file
         if (self.__hilighted_file == f):
             self.set_hilight(idx)
         #self.invalidate_item(idx)
 
+        # prerender some of the items now, some later; this should
+        # distribute CPU load and disk IO
+        #if (l % 3 == 0):
+        #    item.render_at(None, 0, 0)
+        #else:
+        self.__items_to_prerender.append(item)
+
