@@ -5,6 +5,7 @@ from ui.TitleBar import TitleBar
 import platforms
 from theme import theme
 
+import time
 try:
     import gtk
 except:
@@ -82,6 +83,10 @@ class Window(Widget):
                 self.__window = hildon.Window()
                 #self.__window.set_menu(self.__menu)
                 self.__window.fullscreen()
+            elif (platforms.MEEGO_WETAB):
+                self.__window = gtk.Window(gtk.WINDOW_TOPLEVEL)
+                self.__window.set_decorated(False)
+                self.__window.maximize()
             elif (platforms.MEEGO_NETBOOK):
                 self.__window = gtk.Window(gtk.WINDOW_TOPLEVEL)
                 self.__window.set_decorated(False)
@@ -103,6 +108,10 @@ class Window(Widget):
                 self.__window.fullscreen()
                 # hide some ugly separator :)
                 #self.__window.vbox.get_children()[0].hide()
+            elif (platforms.MEEGO_WETAB):
+                self.__window = gtk.Window()
+                self.__window.maximize()
+                self.__window.set_decorated(False)
             elif (platforms.MEEGO_NETBOOK):
                 self.__window = gtk.Dialog()
                 self.__window.set_decorated(False)
@@ -249,6 +258,17 @@ class Window(Widget):
         self.set_flags(new_flags)
 
 
+    def get_flag(self, flag):
+        """
+        Returns the current value of a window flag.
+        @since: 2010.10.16
+        
+        @param flag: the flag
+        """
+        
+        return (self.__flags & flag)
+
+
     """
     def get_size(self):
     
@@ -441,21 +461,26 @@ class Window(Widget):
     def _update_flag(self, flag, value):
 
         if (flag == windowflags.FULLSCREEN):
+            self.__vidscreen.set_size_request(-1, -1)
+            now = time.time()
+            while (gtk.events_pending() and time.time() < now + 1):
+                gtk.main_iteration(False)
+                
             if (platforms.MAEMO5):
                 if (value):
                     self.__window.fullscreen()
                 else:
                     self.__window.unfullscreen()
+
             else:
                 if (self.__title_bar):
                     self.__title_bar.set_visible(not value)
-                    #if (value):
-                    #    self.__window.fullscreen()
-                    #else:
-                    #    self.__window.unfullscreen()
-                    self.render()
-            
-        
+                    if (value):
+                        self.__window.fullscreen()
+                    else:
+                        self.__window.unfullscreen()
+                    #self.render()
+                    
         elif (flag == windowflags.ASR):
             if (platforms.MAEMO5):
                 if (value):
@@ -471,6 +496,12 @@ class Window(Widget):
                     self.__set_portrait_property("_HILDON_PORTRAIT_MODE_REQUEST", 1)
                 else:
                     self.__unset_portrait_property("_HILDON_PORTRAIT_MODE_REQUEST")
+            elif (platforms.MAEMO4):
+                from utils import xrandr
+                if (value):
+                    xrandr.set_orientation(xrandr.RIGHT)
+                else:
+                    xrandr.set_orientation(xrandr.NORMAL)
 
         elif (flag == windowflags.BUSY):
             if (platforms.MAEMO5):
@@ -685,9 +716,10 @@ class Window(Widget):
 
     def show_video_overlay(self, x, y, w, h):
     
+        self.__vidscreen.hide()
         self.__layout.move(self.__vidscreen, x, y)
         self.__vidscreen.set_size_request(w, h)
-        self.__vidscreen.show()
+        gobject.idle_add(self.__vidscreen.show)
         
         return self.__vidscreen.window.xid
 
