@@ -31,6 +31,9 @@ class ItemView(Widget):
         self.__cursor_pos = -1
         self.__hilighted_pos = -1
         
+        # filtering function
+        self.__filter_func = None
+        self.__filter_cache = []
     
         Widget.__init__(self)
 
@@ -70,6 +73,7 @@ class ItemView(Widget):
         self.__hilighted_pos = new_set.hilight
         
         self.__current_set = set_id
+        self.__filter_cache = []
 
 
     def clear_items(self):
@@ -80,6 +84,7 @@ class ItemView(Widget):
         self.__items = []
         self.__cursor_pos = -1
         self.__hilighted_pos = -1
+        self.__filter_cache = []
                 
         
     def append_item(self, item):
@@ -90,6 +95,7 @@ class ItemView(Widget):
         """
     
         self.__items.append(item)
+        self.__filter_cache = []
         
         
     def insert_item(self, item, before_position):
@@ -113,6 +119,7 @@ class ItemView(Widget):
         """
     
         self.__items[position] = item
+        self.__filter_cache = []
 
 
     def remove_item(self, position):
@@ -123,6 +130,7 @@ class ItemView(Widget):
         """
     
         self.__items.pop(position)
+        self.__filter_cache = []
 
 
     def shift_item(self, pos, amount):
@@ -134,9 +142,28 @@ class ItemView(Widget):
         @param amount: shifting amount
         """
         
-        item = self.__items.pop(pos)
-        self.__items.insert(pos + amount, item)
-        self.emit_event(self.EVENT_ITEM_SHIFTED, pos, amount)
+        if (self.__filter_func):
+            items = self.get_items()
+            item = items[pos]
+            other_item = items[pos + amount]
+
+            # pos is on filtered list, find origpos
+            orig_pos = self.__items.index(item)
+            orig_amount = self.__items.index(other_item) - orig_pos
+        else:
+            orig_pos = pos
+            orig_amount = amount
+
+        item = self.__items.pop(orig_pos)
+        self.__items.insert(orig_pos + orig_amount, item)
+
+        # manipulate the cache instead of invalidating; this will perform better
+        # with looooong lists
+        if (self.__filter_func and self.__filter_cache):
+            item = self.__filter_cache.pop(pos)
+            self.__filter_cache.insert(pos + amount, item)
+            
+        self.emit_event(self.EVENT_ITEM_SHIFTED, orig_pos, orig_amount)
 
 
     def count_items(self):
@@ -146,7 +173,7 @@ class ItemView(Widget):
         @return: number of items
         """
         
-        return len(self.__items)
+        return len(self.get_items())
         
        
     def get_items(self):
@@ -156,18 +183,42 @@ class ItemView(Widget):
         @return: list of items
         """
     
-        return self.__items[:]
+        if (self.__filter_func):
+            if (not self.__filter_cache):
+                self.__filter_cache = [ item for item in self.__items
+                                        if self.__filter_func(item) ]
+            return self.__filter_cache
+        else:
+            return self.__items[:]
         
         
     def get_item(self, pos):
         """
         Returns the item at the given position in the current set.
+        Returns None if the position does not exist.
         
         @param pos: position
         @return: the item at that position
         """
     
-        return self.__items[pos]
+        items = self.get_items()
+        if (pos < len(items)):
+            return items[pos]
+        else:
+            return None
+
+
+    def set_filter(self, filter_func = None):
+        """
+        Sets a filter function to filter out items.
+        Filter function has signature: filter_func(item)
+        @since: 2010.09.14
+        
+        @param filter_func: filtering function or None for no filter
+        """
+        
+        self.__filter_func = filter_func
+        self.__filter_cache = []
 
 
     def set_hilight(self, pos):
