@@ -2,6 +2,7 @@
 Lightweight logging module with formatted output. Logs are written to C{stdout}.
 """
 
+import sys
 import time
 import traceback
 
@@ -16,6 +17,8 @@ INFO = 3
 """log level: log errors, warnings, and info"""
 DEBUG = 4
 """log level: log errors, warnings, info, and debugging"""
+PROFILE = 5
+"""log level: log errors, warnings, info, debugging, and profiling"""
 
 
 _level = ERROR
@@ -72,6 +75,7 @@ def _log(ltype, s):
         else:
             print ">   " + line
     #end for
+    sys.stdout.flush()
 
 
 def error(msg, *args):
@@ -122,6 +126,17 @@ def debug(msg, *args):
     _log("DEBUG  ", msg)
 
 
+def profile(t, msg, *args):
+    """
+    @since: 2010.10.09
+    """
+    
+    if (_level < PROFILE): return
+    if (args):
+        msg = msg % args
+    _log("PROFILE (%0.4fs)" % (time.time() - t), msg)
+
+
 def stacktrace():
     """
     Returns the current stack trace.
